; ****************************************************************
; *	                PIC12F675				 *
; *                    ----\/----				 *
; *      V+---------- |1        8| GND				 *
; *            ---GP5 |2        7| GP0---			 *
; *            ---GP4 |3        6| GP1---			 *
; *   RESET    ---GP3 |4        5| GP2---			 *
; *                    ----------			         *
; ****************************************************************

    processor 12F675
    #include <P12F675.INC>
    __config 0x3F84
;   _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_ON 
;   & _WDT_OFF & _INTRC_OSC_NOCLKOUT 



;**************************************************************
   		cblock	0x20
	tmr1var1	; length of 1 dot counter
	tmr1var2        ; length of 1 dash counter
	tmr1var3        ; length register counter  
	counter
    		endc
;**************************************************************
    Org 0x0000

    CLRF 	tmr1var3
    MOVLW 	0x1E     	; b'00011110' 
    MOVWF 	GPIO      	; GPIO initial state
    MOVLW 	0x1F           	; b'00011111' 
    MOVWF 	CMCON          	; comparator OFF / CIN+ on GP0 /
    MOVLW 	0x30           	; b'00110000' 
    MOVWF 	T1CON          	; set prescaler 1:8

    BSF 	STATUS,RP0   	; 	Bank1
    MOVLW 	0x0F           	; b'00001111' GPIO,4&5 OUT
    MOVWF 	TRISIO       	;
    CLRF 	OPTION_REG    	;   
    MOVLW 	0x06           	; b'00000110' PULL-UP GPIO,1&2
    MOVWF 	WPU           
    CLRF 	INTCON          ; disable all interrupts 
    call	0x3FF		; ( only flags work)
    MOVWF 	OSCCAL
    CLRF 	ANSEL		; digital I/O
    MOVLW 	0x8F           	; b'10001111' = 3,6V
    MOVWF 	VRCON		; comparator voltage reference value 
    BCF 	STATUS,RP0      ;  	Bank0

    BCF 	ADCON0,0    	; A/D control register OFF

    CALL 	delay1
    CALL 	chk_speed     	; Check speed
    CALL 	delay1
    CALL 	chk_speed     	; Check speed	
;**************************************************************

start
    BTFSC 	GPIO,3         	; Check Speed Button
    GOTO	paddle1
    CALL 	chk_speed     	; Check speed	
paddle1
    BTFSC 	GPIO,1         	; Check Paddle 1
    GOTO 	paddle2		
;************************ DASH *** 150 cycles for slow speed ***
    BSF 	GPIO,5       	; Set OUT ON
    MOVF 	tmr1var2,W      ; load length
    MOVWF 	tmr1var3	; of 1 dash	
loop11
    BCF 	GPIO,4          ; Set AUDIO ON
    CALL 	delay1          ; 0,65ms : half period for 750Hz
    BSF 	GPIO,4          ; Set AUDIO OFF
    CALL 	delay1          ; 0,65ms : half period for 750Hz
    DECFSZ 	tmr1var3,F	; check if length counter exceed
    GOTO 	loop11		; cycling if not
    BCF 	GPIO,5          ; Set OUT OFF
				; pause procedure
    MOVF 	tmr1var1,W	; load length	
    MOVWF 	tmr1var3	; of pause = 1 dot		 
loop12		
    NOP		
    CALL 	delay1		; 0,65ms
    NOP				; every 0,65 ms during pause cycle
    CALL 	delay1		; 0,65ms
    DECFSZ 	tmr1var3,F	; check if length counter exceed
    GOTO 	loop12		; cycling if not

;**************************************************************

paddle2
    BTFSC 	GPIO,2         	; Check Paddle 2
    GOTO	start	
;************************* DOT *** 50 cycles for slow speed ***
    BSF 	GPIO,5         	; Set OUT ON
    MOVF 	tmr1var1,W      ; load length
    MOVWF 	tmr1var3	; of 1 dot
loop21	
    BCF 	GPIO,4       	; Set AUDIO ON
    CALL 	delay1		; 0,65ms : half period for 750Hz
    BSF 	GPIO,4         	; Set AUDIO OFF
    CALL 	delay1		; 0,65ms : half period for 750Hz
    DECFSZ 	tmr1var3,F      ; check if length counter exceed
    GOTO 	loop21          ; cycling if not
    BCF 	GPIO,5         	; Set OUT OFF   
				; pause procedure
    MOVF 	tmr1var1,W	; load length
    MOVWF 	tmr1var3	; of pause = 1 dot
loop22	
    NOP
    CALL 	delay1		; 0,65ms
    NOP				; every 0,65 ms during pause cycle
    CALL 	delay1		; 0,65ms
    DECFSZ 	tmr1var3,F	; check if length counter exceed
    GOTO 	loop22		; cycling if not
    GOTO 	start		; 

;**************************************************************

chk_speed 
    
    BCF 	GPIO,0          ; 
    BSF 	STATUS,RP0   	; Bank1
    BCF 	TRISIO,0        ; Set GPIO,0 as OUTPUT 
    NOP				; Keep GPIO,0 LOW for 2us
    NOP				; to discharge capasitor 
    BSF 	TRISIO,0        ; Set GPIO,0 as INPUT/start charging  
    BCF 	STATUS,RP0      ; Bank0

    BCF 	CMCON,0         ; comparator ON
    CALL 	delay2		; 14us delay to charge capasitor
    MOVF 	CMCON,F   	; set or clear Z flag
    BCF 	PIR1,3          ; clear comparator interrupt flag
    BSF 	T1CON,0         ; Start Timer1
loop3
    BTFSS 	PIR1,3         	; Check comparator interrupt flag
    GOTO 	loop3		; Wait for the flag
    BCF 	T1CON,0       	; Stop Timer1
    BSF 	CMCON,0       	; comparator OFF

				; load number of cycles to charge capasitor
    MOVF 	TMR1L,W         ; into counters of dots and dashs length
    MOVWF 	tmr1var1        ; counter of length of 1 dot
    MOVWF 	tmr1var2	; counter of length of 1 dash
    ADDWF 	tmr1var2,F      ; 1 dash = 1 dot + 1 dot
    ADDWF 	tmr1var2,F      ; 1 dash = 2 dots + 1 dot = 3 dots
    CLRF 	TMR1L           ; clear TMR1 register
          
    RETURN

;**************************************************************

delay1                   	; 650 cycles = 0,65 ms
    MOVLW 	0xD8           	; d'216'
    MOVWF 	counter
loop1
    DECFSZ 	counter,F
    GOTO 	loop1

    RETURN
;**************************************************************

delay2			 	; 14 cycles = 14 us
    MOVLW 	0x03           	; d'003'
    MOVWF 	counter
loop2
    DECFSZ 	counter,F
    GOTO 	loop2

    RETURN

;**************************************************************

    End
